@@ -1,4 +1,4 @@
-(* $Id: pxp_dtd.ml,v 1.25 2003/06/20 15:14:13 gerd Exp $
+(* $Id: pxp_dtd.ml 697 2004-08-21 21:02:46Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -27,6 +27,8 @@ type validation_record =
 ;;
 
 
+(* class type? *)
+
 class namespace_manager =
 object (self)
     val uri_of_prefix = Hashtbl.create 10  (* not unique *)
@@ -37,7 +39,8 @@ object (self)
       ignore(self # add_namespace "xml" "http://www.w3.org/XML/1998/namespace")
 
     method add_uri (np:string) (uri:string) =
-      if not (Hashtbl.mem uri_of_prefix np) then raise Not_found;
+      if not (Hashtbl.mem uri_of_prefix np) then 
+	raise(Namespace_prefix_not_managed np);
       try
 	let np' = Hashtbl.find prefix_of_uri uri in
 	if np <> np' then
@@ -82,22 +85,132 @@ object (self)
 	      (* prefix = "": make sure that such a prefix is never added *)
 
     method get_primary_uri normprefix =
-      Hashtbl.find primary_uri_of_prefix normprefix
+      try
+	Hashtbl.find primary_uri_of_prefix normprefix
+      with
+	  Not_found -> 
+	    raise(Namespace_prefix_not_managed normprefix)
 
     method get_uri_list normprefix =
       Hashtbl.find_all uri_of_prefix normprefix
 
     method get_normprefix uri =
-      Hashtbl.find prefix_of_uri uri
+      try
+	Hashtbl.find prefix_of_uri uri
+      with
+	  Not_found ->
+	    raise(Namespace_not_managed uri)
 
     method iter_namespaces f =
       Hashtbl.iter 
 	(fun p uri -> f p)
 	primary_uri_of_prefix
+
+    method as_declaration =
+      let l = ref [] in
+      Hashtbl.iter 
+	(fun p uri -> l := (p, uri) :: !l)
+	primary_uri_of_prefix;
+      !l
+      
   end
 ;;
 
 
+let create_namespace_manager () = new namespace_manager;;
+
+
+class type namespace_scope =
+object
+  method namespace_manager : namespace_manager
+  method parent_scope : namespace_scope option
+  method declaration : (string * string) list
+  method effective_declaration : (string * string) list
+  method display_prefix_of_uri : string -> string
+  method display_prefix_of_normprefix : string -> string
+  method uri_of_display_prefix : string -> string
+  method normprefix_of_display_prefix : string -> string
+end
+;;
+
+
+
+module StrSet = Set.Make(String);;
+
+class namespace_scope_impl mng parent_opt decl : namespace_scope =
+object(self)
+  method namespace_manager = mng
+  method parent_scope = parent_opt
+  method declaration = decl
+
+  method effective_declaration =
+    let rec collect visible d s =
+      match d with
+	| ("", "") :: d' ->
+	    if StrSet.mem "" visible then
+	      collect visible d' s  (* no effect *)
+	    else
+	      collect (StrSet.add "" visible) d' s  (* hide inner default *)
+	| (dp, uri) :: d' ->
+	    if StrSet.mem dp visible then
+	      collect visible d' s
+	    else
+	      (dp, uri) :: collect (StrSet.add dp visible) d' s
+	| [] ->
+	    ( match s # parent_scope with
+		  Some s' ->
+		    collect visible s'#declaration s'
+		| None ->
+		    []
+	    )
+    in
+    collect StrSet.empty self#declaration (self : #namespace_scope :> namespace_scope)
+
+  method display_prefix_of_uri uri =
+    try
+      fst(List.find (fun (p,u) -> u = uri) decl)
+    with
+	Not_found ->
+	  ( match parent_opt with
+		Some pa -> pa # display_prefix_of_uri uri
+	      | None    -> raise(Namespace_not_in_scope uri)
+	  )
+
+  method display_prefix_of_normprefix np =
+    let uris = mng # get_uri_list np in
+    if uris = [] then raise(Namespace_prefix_not_managed np);
+    try
+      fst(List.find (fun (p,u) -> List.mem u uris) decl)
+    with
+	Not_found ->
+	  ( match parent_opt with
+		Some pa -> pa # display_prefix_of_normprefix np
+	      | None    -> raise(Namespace_not_in_scope
+				   (List.hd(List.rev uris)))
+	  )
+
+  method uri_of_display_prefix dp =
+    try
+      List.assoc dp decl
+    with
+	Not_found -> 
+	  ( match parent_opt with
+		Some pa -> pa # uri_of_display_prefix dp
+	      | None    -> raise Not_found
+	  )
+
+  method normprefix_of_display_prefix dp =
+    let uri = self # uri_of_display_prefix dp in
+    mng # get_normprefix uri
+
+end
+;;
+
+
+let create_namespace_scope ?parent ?(decl = []) mng =
+  new namespace_scope_impl mng parent decl ;;
+
+
 class dtd  ?swarner the_warner init_encoding =
   object (self)
     val mutable root = (None : string option)
@@ -107,7 +220,7 @@ class dtd  ?swarner the_warner init_encoding =
     val warner       = (the_warner : collect_warnings)
     val swarner      = (swarner : symbolic_warnings option)
     val encoding     = init_encoding
-    val lexerset     = Pxp_lexers.get_lexer_set init_encoding
+    val lfactory     = Pxp_lexers.get_lexer_factory init_encoding
 
     val elements     = (Str_hashtbl.create 100 : dtd_element Str_hashtbl.t)
     val gen_entities = (Str_hashtbl.create 100 : (entity * bool) Str_hashtbl.t)
@@ -146,6 +259,8 @@ class dtd  ?swarner the_warner init_encoding =
 
     method encoding = encoding
 
+    method lexer_factory = lfactory
+
     method warner = warner
 
     method swarner = swarner
@@ -215,7 +330,7 @@ class dtd  ?swarner the_warner init_encoding =
 	if List.mem name [ "lt"; "gt"; "amp"; "quot"; "apos" ] then begin
 	  (* These are allowed to be declared several times *)
 	  let (rt,_) = en # replacement_text in
-	  let toks = tokens_of_content_string lexerset rt in
+	  let toks = tokens_of_content_string lfactory rt in
 	  try
 	    begin match toks with
 	      [CRef 60]       -> if name <> "lt"   then raise Not_found
@@ -281,13 +396,12 @@ class dtd  ?swarner the_warner init_encoding =
 		  "optional-element-and-notation-declarations" ->
 		    self # allow_arbitrary
 		| "optional-attribute-declarations" ->
-		    let lexers = Pxp_lexers.get_lexer_set encoding in
 		    let el_string = 
 		      try List.assoc "elements" atts
 		      with Not_found ->
 			raise(Error("Missing `elements' attribute for pxp:dtd"))
 		    in
-		    let el = split_attribute_value lexers el_string in
+		    let el = split_attribute_value lfactory el_string in
 		    List.iter
 		      (fun e_name ->
 			 let e =
@@ -316,7 +430,7 @@ class dtd  ?swarner the_warner init_encoding =
 			    raise(Error("Cannot do pxp:dtd instruction: namespaces not enabled"))
 			| Some m ->
 			    ( try m # add_uri prefix uri
-			      with Not_found ->
+			      with Namespace_prefix_not_managed _ ->
 				m # add_namespace prefix uri
 			    )
 		    )
@@ -598,7 +712,7 @@ and dtd_element the_dtd the_name =
   object (self)
     val dtd = (the_dtd : dtd)
     val name = the_name
-    val lexerset = Pxp_lexers.get_lexer_set (the_dtd # encoding)
+    val lfactory = the_dtd # lexer_factory
     val mutable content_model = Unspecified
     val mutable content_model_validated = false
     val mutable content_dfa = lazy None
@@ -716,7 +830,7 @@ and dtd_element the_dtd the_name =
 		(* i.e. adefault matches D_default or D_fixed *)
 	    | Some s ->
 		atype <> A_cdata &&
-		normalization_changes_value lexerset atype s
+		normalization_changes_value lfactory atype s
 	)
       with
 	  Not_found ->
@@ -766,9 +880,9 @@ and dtd_element the_dtd the_name =
 		     match d with
 			 (D_required | D_implied) -> Implied_value
 		       | D_default v ->
-			   value_of_attribute lexerset dtd n t v
+			   value_of_attribute lfactory dtd n t v
 		       | D_fixed v ->
-			   value_of_attribute lexerset dtd n t v
+			   value_of_attribute lfactory dtd n t v
 		   in
 
 		   init_att_vals.( !k ) <- (n, init_val);
@@ -1001,7 +1115,7 @@ and dtd_element the_dtd the_name =
 	     let check v =
 	       let lexical_error() =
 		 lazy (raise(Validation_error("Default value for attribute `" ^ n ^ "' is lexically malformed"))) in
-	       check_attribute_value_lexically lexerset (lexical_error()) t v;
+	       check_attribute_value_lexically lfactory (lexical_error()) t v;
 	       begin match t with
 		   (A_entity|A_entities) ->
 		     List.iter
@@ -1014,7 +1128,7 @@ and dtd_element the_dtd the_name =
 -- This is checked anyway when the attribute value is normalized
 *)
 		       )
-		       (split_attribute_value lexerset v)
+		       (split_attribute_value lfactory v)
 		 | A_notation nl ->
 		     if not (List.mem v nl) then
 		       raise(Validation_error("Illegal default value for attribute `" ^ n ^ "' in declaration for element `" ^ name ^ "'"));
@@ -1136,9 +1250,9 @@ object (self)
       wms "?>";
 
     method parse_pxp_option =
-      let lexers = get_lexer_set encoding in
+      let lfactory = get_lexer_factory encoding in
       try
-	let toks = tokens_of_xml_pi lexers value in   (* may raise WF_error *)
+	let toks = tokens_of_xml_pi lfactory value in (* may raise WF_error *)
 	begin match toks with
 	    (Pro_name option_name) :: toks' ->
 	      let atts = decode_xml_pi toks' in       (* may raise WF_error *)
@@ -1153,6 +1267,10 @@ object (self)
   end
 ;;
 
+let create_dtd ?swarner ?(warner = new drop_warnings) enc =
+  new dtd ?swarner warner enc ;;
+
+
 type source =
     Entity of ((dtd -> Pxp_entity.entity) * Pxp_reader.resolver)
   | ExtID of (ext_id * Pxp_reader.resolver)
@@ -1195,179 +1313,13 @@ module Entity = struct
                                  ~name ~xid ~resolver dtd
       | Entity(make,resolver) ->
 	  make dtd  (* resolver ignored *)
+
+  let entity_id ent = (ent :> < >)
+
+  class fake = object end
+
+  let create_entity_id () = new fake
+
 end
 
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_dtd.ml,v $
- * Revision 1.25  2003/06/20 15:14:13  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.24  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.23  2003/03/23 21:43:37  gerd
- * 	Preserving the correct order of processing instructions
- * (Blair Zajac)
- *
- * Revision 1.22  2003/01/21 00:19:18  gerd
- * 	Support for resolver_id.
- *
- * Revision 1.21  2002/03/10 23:39:28  gerd
- * 	Extended the Entity module
- *
- * Revision 1.20  2001/12/03 23:45:55  gerd
- * 	new method [write_ref]
- *
- * Revision 1.19  2001/07/02 23:21:40  gerd
- * 	Added the Entity module.
- *
- * Revision 1.18  2001/06/29 13:57:30  gerd
- * 	Weakened the xml:space check.
- *
- * Revision 1.17  2001/06/08 01:15:46  gerd
- * 	Moved namespace_manager from Pxp_document to Pxp_dtd. This
- * makes it possible that the DTD can recognize the processing instructions
- * <?pxp:dtd namespace prefix="..." uri="..."?>, and add the namespace
- * declaration to the manager.
- *
- * Revision 1.16  2001/06/07 22:48:38  gerd
- * 	Improvement: 'write' writes sorted attributes. This makes
- * many regression tests simpler.
- *
- * Revision 1.15  2001/04/22 14:14:41  gerd
- * 	Updated to support private IDs.
- *
- * Revision 1.14  2000/10/01 19:47:19  gerd
- * 	Using Str_hashtbl instead of Hashtbl.
- *
- * Revision 1.13  2000/09/22 22:54:30  gerd
- * 	Optimized the attribute checker (internal_init of element
- * nodes). The validation_record has now more fields to support
- * internal_init.
- *
- * Revision 1.12  2000/09/16 22:40:50  gerd
- * 	Bug processing processing instructions: Method
- * pinstr_names returned wrong results; method write wrote
- * the wrong instructions.
- *
- * Revision 1.11  2000/09/09 16:41:32  gerd
- * 	New type validation_record.
- *
- * Revision 1.10  2000/08/18 21:18:45  gerd
- * 	Updated wrong comments for methods par_entity and gen_entity.
- * These can raise WF_error and not Validation_error, and this is the
- * correct behaviour.
- *
- * Revision 1.9  2000/07/25 00:30:01  gerd
- * 	Added support for pxp:dtd PI options.
- *
- * Revision 1.8  2000/07/23 02:16:34  gerd
- * 	Support for DFAs.
- *
- * Revision 1.7  2000/07/16 17:50:01  gerd
- * 	Fixes in 'write'
- *
- * Revision 1.6  2000/07/16 16:34:41  gerd
- * 	New method 'write', the successor of 'write_compact_as_latin1'.
- *
- * Revision 1.5  2000/07/14 13:56:48  gerd
- * 	Added methods id_attribute_name and idref_attribute_names.
- *
- * Revision 1.4  2000/07/09 00:13:37  gerd
- * 	Added methods gen_entity_names, par_entity_names.
- *
- * Revision 1.3  2000/07/04 22:10:55  gerd
- * 	Update: collect_warnings -> drop_warnings.
- * 	Update: Case ext_id = Anonymous.
- *
- * Revision 1.2  2000/06/14 22:19:06  gerd
- * 	Added checks such that it is impossible to mix encodings.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- *
- * Revision 1.18  2000/05/28 17:24:55  gerd
- * 	Bugfixes.
- *
- * Revision 1.17  2000/05/27 19:21:25  gerd
- * 	Implemented the changes of rev. 1.10 of markup_dtd.mli.
- *
- * Revision 1.16  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.15  2000/05/14 21:50:07  gerd
- * 	Updated: change in internal_entity.
- *
- * Revision 1.14  2000/05/06 23:08:46  gerd
- * 	It is possible to allow undeclared attributes.
- *
- * Revision 1.13  2000/05/01 20:42:46  gerd
- *         New method write_compact_as_latin1.
- *
- * Revision 1.12  2000/05/01 15:16:57  gerd
- * 	The errors "undeclared parameter/general entities" are
- * well-formedness errors, not validation errors.
- *
- * Revision 1.11  2000/03/11 22:58:15  gerd
- * 	Updated to support Markup_codewriter.
- *
- * Revision 1.10  2000/01/20 20:53:47  gerd
- * 	Changed such that it runs with Markup_entity's new interface.
- *
- * Revision 1.9  1999/11/09 22:15:41  gerd
- * 	Added method "arbitrary_allowed".
- *
- * Revision 1.8  1999/09/01 22:52:22  gerd
- * 	If 'allow_arbitrary' is in effect, no validation happens anymore.
- *
- * Revision 1.7  1999/09/01 16:21:24  gerd
- * 	Added several warnings.
- * 	The attribute type of "xml:space" is now strictly checked.
- *
- * Revision 1.6  1999/08/15 20:34:21  gerd
- * 	Improved error messages.
- * 	Bugfix: It is no longer allowed to create processing instructions
- * with target "xml".
- *
- * Revision 1.5  1999/08/15 02:20:16  gerd
- * 	New feature: a DTD can allow arbitrary elements.
- *
- * Revision 1.4  1999/08/15 00:21:39  gerd
- * 	Comments have been updated.
- *
- * Revision 1.3  1999/08/14 22:12:52  gerd
- *         Several functions have now a "warner" as argument which is
- * an object with a "warn" method. This is used to warn about characters
- * that cannot be represented in the Latin 1 alphabet.
- * 	Bugfix: if two general entities with the same name are definied,
- * the first counts, not the second.
- *
- * Revision 1.2  1999/08/11 14:56:35  gerd
- * 	Declaration of the predfined entities {lt,gt,amp,quot,apos}
- * is no longer forbidden; but the original definition cannot be overriddden.
- * 	TODO: If these entities are redeclared with problematic values,
- * the user should be warned.
- *
- * Revision 1.1  1999/08/10 00:35:51  gerd
- * 	Initial revision.
- *
- *
- *)
