@@ -1,4 +1,4 @@
-(* $Id: pxp_dtd.mli,v 1.20 2003/06/20 19:33:05 gerd Exp $
+(* $Id: pxp_dtd.mli 697 2004-08-21 21:02:46Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -52,7 +52,7 @@ class namespace_manager :
   (* This class manages mappings from URIs to normalized prefixes. For every
    * namespace a namespace_manager object contains a set of mappings
    * uri1 |-> np, uri2 |-> np, ..., uriN |-> np.
-   * The normalized prefix np is characterical of the namespace, and
+   * The normalized prefix np is characterstic of the namespace, and
    * identifies the namespace uniquely.
    * The first URI uri1 is the primary URI, the other URIs are aliases.
    * The following operations are supported:
@@ -78,10 +78,14 @@ class namespace_manager :
     method add_uri : string -> string -> unit
       (* add_uri np uri: adds uri as alias URI to the namespace identified
        * by the normprefix np (see above for detailed semantics). The method
-       * raises Not_found if the normprefix np is unknown to the object,
+       * raises Namespace_prefix_not_managed if the normprefix np is unknown
+       * to the object,
        * and it fails (Namespace_error) if the uri is member of a
        * different namespace. Nothing happens if the uri is already member
        * of the namespace np.
+       *
+       * CHANGE IN PXP 1.2: Using exception Namespace_prefix_not_managed
+       * instead of Not_found.
        *)
     method add_namespace : string -> string -> unit
       (* add_namespace np uri: adds a new namespace to the object. The
@@ -102,8 +106,12 @@ class namespace_manager :
        * the normprefix.
        *)
     method get_primary_uri : string -> string
-      (* Return the primary URI for a normprefix, or raises Not_found.
-       * get_uri "" raises always Not_found.
+      (* Return the primary URI for a normprefix, or raises
+       * Namespace_prefix_not_managed. get_uri "" raises always this
+       * exception.
+       *
+       * CHANGE IN PXP 1.2: Using exception Namespace_prefix_not_managed
+       * instead of Not_found.
        *)
     method get_uri_list : string -> string list
       (* Return all URIs for a normprefix, or [] if the normprefix is
@@ -111,12 +119,22 @@ class namespace_manager :
        * returned list is the primary URI.
        *)
     method get_normprefix : string -> string
-      (* Return the normprefix for a URI, or raises Not_found *)
+      (* Return the normprefix for a URI, or raises 
+       * Namespace_not_managed.
+       *
+       * CHANGE IN PXP 1.2: Using exception Namespace_not_managed
+       * instead of Not_found.
+       *)
     method iter_namespaces : (string -> unit) -> unit
       (* Iterates over all namespaces contained in the object, and
        * calls the passed function for every namespace. The argument of the
        * invoked function is the normprefix of the namespace.
        *)
+    method as_declaration : (string * string) list
+      (* Returns the list of normprefixes and primary URIs. Useful
+       * to create the corresponding namespace scope, e.g.
+       * new namespace_scope_impl mng None (mng#as_declaration)
+       *)
 
     (* Encodings: prefixes and URIs are always encoded in the default
      * encoding of the document
@@ -125,6 +143,99 @@ class namespace_manager :
 ;;
 
 
+val create_namespace_manager : unit -> namespace_manager
+  (* Preferred way of creating a namespace_manager *)
+
+
+(** The recursive class type [namespace_scope] represents the original
+ * namespace declarations found in the XML text. A single [namespace_scope]
+ * object contains a list of declared namespaces
+ * {[ [ (dp1, uri1); (dp2; uri2); ... ] ]}
+ * corresponding to the "xmlns"-type declarations found in a single
+ * XML element:
+ * {[ <element xmlns:dp1="uri1" xmlns:dp2="uri2" ... > ]}
+ * For the declaration of a default namespace [xmlns="uri"] the pair
+ * [("",uri)] must be included in the list. The special pair [("","")]
+ * means that the former default namespace is "undeclared".
+ *
+ * Furthermore, the [namespace_scope] object may have a parent 
+ * [namespace_scope], representing the namespace declarations in the
+ * surrounding XML text.
+ *
+ * The [namespace_scope] objects are connected with the [namespace_manager]
+ * to allow translations from the namespace prefixes found in the XML
+ * text (also called "display prefixes" from now on) to the normalized
+ * prefixes stored in the [namespace_manager], and vice versa.
+ *
+ * The [namespace_scope] objects are intentionally immutable in order to
+ * allow memory sharing.
+ *)
+class type namespace_scope =
+object
+  method namespace_manager : namespace_manager
+    (** Returns the [namespace_manager] to which this scope object is
+     * connected
+     *)
+  method parent_scope : namespace_scope option
+    (** Returns the parent object, if any *)
+  method declaration : (string * string) list
+    (** Returns the list of namespace declarations of this scope (i.e.
+     * the declarations in parent objects are not considered). The
+     * list contains pairs [ (display_prefix, uri) ].
+     *)
+  method effective_declaration : (string * string) list
+    (** Returns the list of namespace declarations of this scope and
+     * all parent scopes. The list contains pairs [ (display_prefix, uri) ].
+     * Prefixes hidden by earlier declarations are suppressed in the list
+     *)
+  method display_prefix_of_uri : string -> string
+    (** Translates the URI to the corresponding display prefix as declared
+     * in this object or any parent object. Raises [Namespace_not_in_scope]
+     * when the declaration cannot be found.
+     *)
+  method display_prefix_of_normprefix : string -> string
+    (** Translates the normalized prefix to the corresponding display
+     * prefix as declared in this object or any parent object. Raises
+     * [Namespace_not_in_scope] when the declaration cannot be found, and
+     * [Namespace_prefix_not_managed] when the
+     * normalized prefix is unknown to the namespace manager.
+     *)
+  method uri_of_display_prefix : string -> string
+    (** Translates the display prefix to the corresponding URI as
+     * declared in this object or any parent object. Raises
+     * [Not_found] when the declaration cannot be found.
+     *)
+  method normprefix_of_display_prefix : string -> string
+    (** Translates the display prefix to the corresponding normalized
+     * prefix as declared in this object or any parent object. Raises
+     * [Not_found] when the declaration cannot be found, and
+     * [Namespace_not_managed] when the
+     * namespace manager does not know the namespace.
+     *)
+end
+;;
+
+
+class namespace_scope_impl : 
+        namespace_manager -> 
+	namespace_scope option ->
+	(string * string) list ->
+	  namespace_scope
+   (** An implementation of [namespace_scope]. New scopes are created by
+    * {[ new namespace_scope_impl mng parent_opt decl ]}
+    * where [mng] is the namespace managaer, [parent_opt] is the optional
+    * parent scope, and [decl] is the declaration list.
+    *)
+
+
+val create_namespace_scope : 
+    ?parent:namespace_scope -> 
+    ?decl:(string * string) list ->
+    namespace_manager ->
+      namespace_scope
+  (* Preferred way of creating a namespace_scope *)
+    
+
 class dtd :
   (* Creation:
    *   new dtd
@@ -167,6 +278,9 @@ class dtd :
     method encoding : Pxp_core_types.rep_encoding
       (* returns the encoding used for character representation *)
 
+    method lexer_factory : Pxp_lexer_types.lexer_factory
+      (** Returns a lexer factory for the character encoding *)
+
 
     method allow_arbitrary : unit
       (* After this method has been invoked, the object changes its behaviour:
@@ -516,6 +630,23 @@ and proc_instruction : string -> string -> Pxp_core_types.rep_encoding ->
 
 (*$-*)
 
+(* ---------------------------------------------------------------------- *)
+
+val create_dtd :
+      ?swarner:Pxp_core_types.symbolic_warnings ->
+      ?warner:Pxp_core_types.collect_warnings -> 
+      Pxp_core_types.rep_encoding ->
+	dtd
+  (* Preferred way of creating a DTD. Example:
+   * let dtd = create_dtd 
+   *             ?swarner:config.swarner
+   *             ~warner:config.warner
+   *             config.encoding
+   *
+   * See also Pxp_dtd_parser.create_empty_dtd.
+   *)
+
+
 (* ---------------------------------------------------------------------- *)
 
 type source =
@@ -620,141 +751,16 @@ module Entity : sig
 	Pxp_entity.entity
       (* Creates an external entity that reads from the passed source *)
 
+  val entity_id : Pxp_entity.entity -> Pxp_lexer_types.entity_id
+    (* Returns the abstract entity ID *)
+
+  val create_entity_id : unit -> Pxp_lexer_types.entity_id
+    (* Create a new abstract entity ID. This ID can be used whereever
+     * an entity_id is expected but no entity is available.
+     *)
+
 end
 ;;
 
 
 
-(* ======================================================================
- * History:
- * 
- * $Log: pxp_dtd.mli,v $
- * Revision 1.20  2003/06/20 19:33:05  gerd
- * 	Improved docs.
- *
- * Revision 1.19  2003/06/20 15:14:13  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.18  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.17  2003/01/21 00:19:18  gerd
- * 	Support for resolver_id.
- *
- * Revision 1.16  2002/03/10 23:39:28  gerd
- * 	Extended the Entity module
- *
- * Revision 1.15  2001/12/03 23:45:55  gerd
- * 	new method [write_ref]
- *
- * Revision 1.14  2001/07/02 23:21:40  gerd
- * 	Added the Entity module.
- *
- * Revision 1.13  2001/06/08 01:15:47  gerd
- * 	Moved namespace_manager from Pxp_document to Pxp_dtd. This
- * makes it possible that the DTD can recognize the processing instructions
- * <?pxp:dtd namespace prefix="..." uri="..."?>, and add the namespace
- * declaration to the manager.
- *
- * Revision 1.12  2001/06/07 22:49:11  gerd
- * 	Removed: method write_compact_as_latin1
- *
- * Revision 1.11  2000/10/01 19:46:57  gerd
- * 	Changed type of the att_lookup component.
- *
- * Revision 1.10  2000/09/22 22:54:30  gerd
- * 	Optimized the attribute checker (internal_init of element
- * nodes). The validation_record has now more fields to support
- * internal_init.
- *
- * Revision 1.9  2000/09/09 16:41:32  gerd
- * 	New type validation_record.
- *
- * Revision 1.8  2000/08/18 21:18:45  gerd
- * 	Updated wrong comments for methods par_entity and gen_entity.
- * These can raise WF_error and not Validation_error, and this is the
- * correct behaviour.
- *
- * Revision 1.7  2000/07/25 00:30:01  gerd
- * 	Added support for pxp:dtd PI options.
- *
- * Revision 1.6  2000/07/23 02:16:33  gerd
- * 	Support for DFAs.
- *
- * Revision 1.5  2000/07/16 16:34:41  gerd
- * 	New method 'write', the successor of 'write_compact_as_latin1'.
- *
- * Revision 1.4  2000/07/14 13:56:49  gerd
- * 	Added methods id_attribute_name and idref_attribute_names.
- *
- * Revision 1.3  2000/07/09 00:13:37  gerd
- * 	Added methods gen_entity_names, par_entity_names.
- *
- * Revision 1.2  2000/06/14 22:19:06  gerd
- * 	Added checks such that it is impossible to mix encodings.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_dtd.ml:
- *
- * Revision 1.11  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.10  2000/05/27 19:20:38  gerd
- * 	Changed the interfaces for the standalone check: New
- * methods: standalone_declaration, set_standalone_declaration,
- * externally_declared, attribute_violates_standalone_declaration.
- * 	The method set_content_model has been renamed to
- * set_cm_and_extdecl; it now initializes also whether the element
- * has been declared in an external entity.
- * 	Methods add_gen_entity and gen_entity pass an additional
- * boolean argument containing whether the declaration of the
- * general entity happened in an external entity.
- * 	Method add_attribute expects this argument, too, which
- * states whether the declaration of the attribute happened in an
- * external entity.
- *
- * Revision 1.9  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.8  2000/05/06 23:10:26  gerd
- * 	allow_arbitrary for elements, too.
- *
- * Revision 1.7  2000/05/01 20:42:52  gerd
- *         New method write_compact_as_latin1.
- *
- * Revision 1.6  2000/03/11 22:58:15  gerd
- * 	Updated to support Markup_codewriter.
- *
- * Revision 1.5  2000/02/22 02:32:02  gerd
- * 	Updated.
- *
- * Revision 1.4  1999/11/09 22:15:41  gerd
- * 	Added method "arbitrary_allowed".
- *
- * Revision 1.3  1999/09/01 16:21:56  gerd
- * 	"dtd" classes have now an argument that passes a "warner".
- *
- * Revision 1.2  1999/08/15 02:20:23  gerd
- *         New feature: a DTD can allow arbitrary elements.
- *
- * Revision 1.1  1999/08/10 00:35:51  gerd
- * 	Initial revision.
- *
- * 
- *)
