@@ -1,4 +1,4 @@
-(* $Id: pxp_entity.ml,v 1.24 2003/11/04 22:45:07 gerd Exp $
+(* $Id: pxp_entity.ml 707 2004-09-04 17:25:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -76,19 +76,23 @@ type 'entity entity_variables =
       mutable warner : collect_warnings;
       
       mutable encoding : rep_encoding;
-      mutable lexerset : lexer_set;
+      mutable lfactory : lexer_factory;
 
-      mutable lexerset_scan_document : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_document_type : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_content : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_within_tag : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_declaration : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_comment : Lexing.lexbuf -> lexers -> (token * lexers);
-      mutable lexerset_scan_tag_eb : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_tag_eb_att : Lexing.lexbuf -> bool -> (token * lexers);
-      mutable lexbuf : Lexing.lexbuf;
+      mutable l_scan_document : unit -> (token * lexers);
+      mutable l_scan_document_type : unit -> (token * lexers);
+      mutable l_scan_content : unit -> (token * lexers);
+      mutable l_scan_within_tag : unit -> (token * lexers);
+      mutable l_scan_declaration : unit -> (token * lexers);
+      mutable l_scan_comment : unit -> lexers -> (token * lexers);
+      mutable l_scan_tag_eb : unit -> (token * lexers);
+      mutable l_scan_tag_eb_att : unit -> bool -> (token * lexers);
+      mutable lexobj : lexer_obj;
         (* The lexical buffer currently used as character source. *)
 
+      (* Caution: whenever lexobj is changed, the l_* functions must be
+       * updated, too (update_lexobj)
+       *)
+
       mutable prolog : prolog_token list option;
         (* Stores the initial <?xml ...?> token as PI_xml *)
       mutable prolog_pairs : (string * string) list;
@@ -141,25 +145,26 @@ type 'entity entity_variables =
 
 
 let make_variables the_dtd the_name the_swarner the_warner init_encoding =
-  let ls = Pxp_lexers.get_lexer_set init_encoding in
+  let lf = Pxp_lexers.get_lexer_factory init_encoding in
+  let lobj = lf # open_string "" in
   { dtd = (the_dtd : 'entity #preliminary_dtd :> 'entity preliminary_dtd);
     name = the_name;
     warner = the_warner;
     swarner = the_swarner;
     
     encoding = init_encoding;
-    lexerset = ls;
+    lfactory = lf;
 
-    lexerset_scan_document         = ls.scan_document;
-    lexerset_scan_document_type    = ls.scan_document_type;
-    lexerset_scan_content          = ls.scan_content;
-    lexerset_scan_within_tag       = ls.scan_within_tag;
-    lexerset_scan_declaration      = ls.scan_declaration;
-    lexerset_scan_comment          = ls.scan_comment;
-    lexerset_scan_tag_eb           = ls.scan_tag_eb;
-    lexerset_scan_tag_eb_att       = ls.scan_tag_eb_att;
+    l_scan_document         = lobj#scan_document;
+    l_scan_document_type    = lobj#scan_document_type;
+    l_scan_content          = lobj#scan_content;
+    l_scan_within_tag       = lobj#scan_within_tag;
+    l_scan_declaration      = lobj#scan_declaration;
+    l_scan_comment          = lobj#scan_comment;
+    l_scan_tag_eb           = lobj#scan_tag_eb;
+    l_scan_tag_eb_att       = lobj#scan_tag_eb_att;
     
-    lexbuf = Pxp_lexing.from_string "";
+    lexobj = lobj;
     
     prolog = None;
     prolog_pairs = [];
@@ -189,6 +194,19 @@ let make_variables the_dtd the_name the_swarner the_warner init_encoding =
   }
 ;;
 
+let update_lexobj v lobj =
+  v.lexobj <- lobj;
+  v.l_scan_document         <- lobj#scan_document;
+  v.l_scan_document_type    <- lobj#scan_document_type;
+  v.l_scan_content          <- lobj#scan_content;
+  v.l_scan_within_tag       <- lobj#scan_within_tag;
+  v.l_scan_declaration      <- lobj#scan_declaration;
+  v.l_scan_comment          <- lobj#scan_comment;
+  v.l_scan_tag_eb           <- lobj#scan_tag_eb;
+  v.l_scan_tag_eb_att       <- lobj#scan_tag_eb_att;
+;;
+
+
 let update_lines v =
   let n_lines = v.linecount.lines in
   let n_columns = v.linecount.columns in
@@ -201,11 +219,11 @@ let update_content_lines v tok =
       LineEnd _ -> 
 	v.line <- v.line + 1;
 	v.column <- 0;
-    | (PI(_,_)|PI_xml _|Cdata _) ->
-	count_lines v.linecount (Lexing.lexeme v.lexbuf);
+    | (PI(_,_,_)|PI_xml _|Cdata _) ->
+	count_lines v.linecount v.lexobj#lexeme;
 	update_lines v;
     | _ -> 
-	v.column <- v.column + Pxp_lexing.lexeme_len v.lexbuf
+	v.column <- v.column + v.lexobj#lexeme_strlen
 ;;
 
 let update_lines_within_tag v tok =
@@ -223,11 +241,11 @@ let update_lines_within_tag v tok =
 	v.line <- v.line + 1;
 	v.column <- 0;
     | _ -> 
-	v.column <- v.column + Pxp_lexing.lexeme_len v.lexbuf 
+	v.column <- v.column + v.lexobj#lexeme_strlen
 ;;
 
 let update_other_lines v tok =
-  count_lines v.linecount (Lexing.lexeme v.lexbuf);
+  count_lines v.linecount v.lexobj#lexeme;
   update_lines v;
 ;;
 
@@ -279,7 +297,7 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
     method set_counts_as_external =
       v.counts_as_external <- true
 
-    method lexbuf = v.lexbuf
+    method lexer_obj = v.lexobj
 
     method virtual resolver : resolver option
 
@@ -336,7 +354,7 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
 	 *)
 
 
-    method lexbuf = v.lexbuf
+    method lexer_obj = v.lexobj
 
 
     method xml_declaration =
@@ -387,29 +405,29 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
 	    let tok, lex_id' =
 	      match v.lex_id with
 		  Document         -> update_fn := update_other_lines;
-                                      v.lexerset_scan_document v.lexbuf 
+                                      v.l_scan_document () 
 		| Document_type    -> update_fn := update_other_lines;
-                                      v.lexerset_scan_document_type v.lexbuf
-		| Content          -> v.lexerset_scan_content v.lexbuf
+                                      v.l_scan_document_type ()
+		| Content          -> v.l_scan_content ()
 		| Within_tag       -> update_fn := update_lines_within_tag;
-                                      v.lexerset_scan_within_tag v.lexbuf
+                                      v.l_scan_within_tag ()
 		| Within_tag_entry -> if v.generate_attribute_events then (
 		                        (* like Tag_eb: *)
 		                        update_fn := update_lines_within_tag;
-                                        v.lexerset_scan_tag_eb v.lexbuf
+                                        v.l_scan_tag_eb ()
                                       ) else (
 		                        (* like Within_tag: *)
 		                        update_fn := update_lines_within_tag;
-                                        v.lexerset_scan_within_tag v.lexbuf
+                                        v.l_scan_within_tag ()
 		                      )
 		| Declaration      -> update_fn := update_other_lines;
-                                      v.lexerset_scan_declaration v.lexbuf
+                                      v.l_scan_declaration ()
 		| Comment flw_id   -> update_fn := update_other_lines;
-		                      v.lexerset_scan_comment v.lexbuf flw_id
+		                      v.l_scan_comment () flw_id
 		| Tag_eb           -> update_fn := update_lines_within_tag;
-                                      v.lexerset_scan_tag_eb v.lexbuf
+                                      v.l_scan_tag_eb ()
 		| Tag_eb_att b     -> (* keep update_content_lines! *)
-                                      v.lexerset_scan_tag_eb_att v.lexbuf b
+                                      v.l_scan_tag_eb_att () b
 		| Ignored_section  -> assert false
   	                           (* only used by method next_ignored_token *)
 		| Closed           -> (Eof, Closed)
@@ -477,16 +495,18 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
           (* Also normalize CDATA sections *)
 		| Cdata value as cd ->
 		    if v.normalize_newline then 
-		      Cdata(normalize_line_separators v.lexerset value)
+		      Cdata(normalize_line_separators v.lfactory value)
 		    else
 		      cd
 
           (* If there are CRLF sequences in a PI value, normalize them, too *)
-		| PI(name,value) as pi ->
+		| PI(name,value,_) as pi ->
 		    if v.normalize_newline then
-		      PI(name, normalize_line_separators v.lexerset value)
+		      PI(name, 
+			 normalize_line_separators v.lfactory value,
+			 (self :> entity_id))
 		    else
-		      pi
+		      PI(name, value, (self :> entity_id))
          
           (* Attribute values: If they are already normalized, they are turned
 	   * into Attval_nl_normalized. This is detected by other code.
@@ -500,7 +520,7 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
           (* Another CRLF normalization case: Unparsed_string *)
 		| Unparsed_string value as ustr ->
 		    if v.normalize_newline then
-		      Unparsed_string(normalize_line_separators v.lexerset value)
+		      Unparsed_string(normalize_line_separators v.lfactory value)
 		    else
 		      ustr
 
@@ -603,7 +623,7 @@ class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
         if v.lex_id = Closed then
 	  Eof
 	else
-	  let tok, lex_id' = v.lexerset.scan_ignored_section v.lexbuf in
+	  let tok, lex_id' = v.lexobj#scan_ignored_section() in
 	  if v.debug then
 	    prerr_endline ("- Entity " ^ v.name ^ ": " ^ string_of_tok tok ^ " (Ignored)");
 	  update_other_lines v tok;
@@ -731,9 +751,9 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
 
     method set_debugging_mode (_:bool) = ()
 
-    method lexbuf =
+    method lexer_obj =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
-	  : Lexing.lexbuf )
+	  : lexer_obj )
 
     method open_entity ?(gen_att_events:bool option) (_:bool) (_:lexers) =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
@@ -749,9 +769,9 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : (string * bool) )
 
-    method lexbuf =
+    method lexer_obj =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
-	  : Lexing.lexbuf )
+	  : lexer_obj )
 
     method next_token =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
@@ -848,7 +868,7 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
        *)
       if resolver_is_open then
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
-      let lex = 
+      let lex_src = 
 	try
 	  resolver # open_rid (self # resolver_id)
 	with
@@ -863,8 +883,10 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
 			   self # full_name ^ "; reason: " ^ 
 			   string_of_exn e))
       in
+
+      let lexobj = v.lfactory#open_source lex_src in
+      update_lexobj v lexobj;
       resolver_is_open <- true;
-      v.lexbuf  <- lex;
       v.prolog  <- None;
       v.lex_id  <- init_lex_id;
       state <- At_beginning;
@@ -930,7 +952,7 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
        *)
       if resolver_is_open then
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
-      let lex = 
+      let lex_src = 
 	try
 	  resolver # open_rid (self # resolver_id)
 	with
@@ -945,8 +967,10 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
 			   self # full_name ^ "; reason: " ^ 
 			   string_of_exn e))
       in
+
+      let lexobj = v.lfactory#open_source lex_src in
+      update_lexobj v lexobj;
       resolver_is_open <- true;
-      v.lexbuf  <- lex;
       v.prolog  <- None;
       (* arbitrary:    lex_id  <- init_lex_id; *)
       state <- At_beginning;
@@ -954,14 +978,14 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
       v.column <- 0;
       v.at_bof <- true;
       (* First check if the first token of 'lex' is <?xml...?> *)
-      if v.lexerset.detect_xml_pi lex then begin
+      if lexobj#detect_xml_pi() then begin
 	(* detect_xml_pi scans "<?xml" ws+. Read the rest of the XML
 	 * declaration.
 	 *)
-	match v.lexerset.scan_pi_string lex with
+	match lexobj#scan_pi_string() with
 	    Some pi ->
 	      begin match
-		Pxp_lex_aux.scan_pi ("xml " ^ pi) v.lexerset.scan_xml_pi
+		Pxp_lex_aux.scan_pi ("xml " ^ pi) v.lfactory
 	      with
 		  PI_xml pl ->
 		    self # process_xmldecl pl
@@ -978,7 +1002,7 @@ class external_entity the_resolver the_dtd the_name the_swarner the_warner
 	self # process_missing_xmldecl;
       (* Then create the replacement text. *)
       let rec scan_and_expand () =
-	match v.lexerset.scan_dtd_string v.lexbuf with
+	match lexobj#scan_dtd_string() with
 	    ERef n -> "&" ^ n ^ ";" ^ scan_and_expand()
 	  | CRef(-1) -> "\n" ^ scan_and_expand()
 	  | CRef(-2) -> "\n" ^ scan_and_expand()
@@ -1065,9 +1089,9 @@ class internal_entity the_dtd the_name the_swarner the_warner the_literal_value
 
 
     initializer
-    let lexbuf = Pxp_lexing.from_string the_literal_value in
+    let lexobj = v.lfactory#open_string the_literal_value in
     let rec scan_and_expand () =
-      match v.lexerset.scan_dtd_string lexbuf with
+      match lexobj#scan_dtd_string() with
 	  ERef n -> "&" ^ n ^ ";" ^ scan_and_expand()
 	| CRef(-1) -> "\r\n" ^ scan_and_expand()
 	| CRef(-2) -> "\r" ^ scan_and_expand()
@@ -1114,11 +1138,12 @@ class internal_entity the_dtd the_name the_swarner the_warner the_literal_value
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
 
       p_parsed_actually <- force_parsing;
-      v.lexbuf  <- Pxp_lexing.from_string 
+      let lexobj = v.lfactory#open_string
 	             (if is_parameter_entity then
 			(" " ^ replacement_text ^ " ")
 		      else
-			replacement_text);
+			replacement_text) in
+      update_lexobj v lexobj;
       v.prolog  <- None;
       v.lex_id  <- init_lex_id;
       state <- At_beginning;
@@ -1244,7 +1269,7 @@ object (self)
   method counts_as_external = ent # counts_as_external
   method set_counts_as_external : unit = 
     failwith "Pxp_entity.entity_section#set_counts_as_external: not possible";
-  method lexbuf = ent # lexbuf
+  method lexer_obj = ent # lexer_obj
   method resolver = ent # resolver
   method resolver_id = ent # resolver_id
   method open_entity ?(gen_att_events:bool option) (_:bool) (lid:lexers) = 
@@ -1311,296 +1336,3 @@ end
 
 (* class entity_manager: has been moved to Pxp_entity_manager *)
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_entity.ml,v $
- * Revision 1.24  2003/11/04 22:45:07  gerd
- * 	Problems with Lexing under 3.07 fixed
- *
- * Revision 1.23  2003/06/20 15:14:14  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.22  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.21  2003/01/21 00:19:05  gerd
- * 	Support for resolver_id.
- *
- * Revision 1.20  2002/08/31 23:23:43  gerd
- * 	Replaced the three comment lexers by only one unified comment
- * lexer.
- * 	Improved way of detecting the XML declaration at the beginning
- * of external files when computing the replacement text.
- *
- * Revision 1.19  2002/08/05 22:33:33  gerd
- * 	New token LineEnd_att for newline characters inside
- * attribute values (event-based parser).
- *
- * Revision 1.18  2002/08/03 17:52:34  gerd
- * 	Support for event-based parsing of attribute values. There is
- * now a flag gen_att_events that determines whether to use this feature.
- * 	Fix in open_entity of class entity_section: Sets the lex_id.
- *
- * Revision 1.17  2002/07/14 23:03:29  gerd
- * 	New class entity_section.
- *
- * Revision 1.16  2002/03/10 23:39:45  gerd
- * 	ext_id works also for external entities.
- *
- * Revision 1.15  2002/02/20 00:25:23  gerd
- * 	using Pxp_lexing instead of Lexing.
- *
- * Revision 1.14  2001/06/28 22:42:07  gerd
- * 	Fixed minor problems:
- * 	- Comments must be contained in one entity
- * 	- Pxp_document.document is now initialized with encoding.
- *           the DTD encoding may be initialized too late.
- *
- * Revision 1.13  2001/04/22 14:14:41  gerd
- * 	Updated to support private IDs.
- *
- * Revision 1.12  2001/04/22 12:04:55  gerd
- * 	external_entity, method replacement_text: catches errors
- * from pxp_reader and transforms them
- *
- * Revision 1.11  2000/10/01 19:49:51  gerd
- * 	Numerous optimizations in the class "entity".
- *
- * Revision 1.10  2000/09/21 21:28:16  gerd
- * 	New token IgnoreLineEnd: simplifies line counting, and
- * corrects a bug.
- *
- * Revision 1.9  2000/09/17 00:11:22  gerd
- * 	Optimized line numbering.
- *
- * Revision 1.8  2000/09/09 16:39:05  gerd
- * 	Changed comment.
- *
- * Revision 1.7  2000/09/05 21:52:31  gerd
- * 	class internal_entity: Previously, the method open_entity
- * intialized the slot last_token to Eof. This is wrong, because
- * this causes that handle_bof is never called. The slot last_token
- * is now initialized to Bof.
- * 	Critical negative tests: data_jclark_notwf/not-sa/002,
- * data_jclark_notwf/sa/153.xml, data_jclark_notwf/sa/161.xml. The
- * error messages of these tests changed (checked; the new messages
- * are better).
- *
- * Revision 1.6  2000/07/14 13:55:00  gerd
- * 	Cosmetic changes.
- *
- * Revision 1.5  2000/07/09 17:51:50  gerd
- * 	Entities return now the beginning of a token as its
- * position.
- * 	New method 'position' for entity_manager.
- *
- * Revision 1.4  2000/07/09 01:05:04  gerd
- * 	Exported methods 'ext_id' and 'notation' anyway.
- *
- * Revision 1.3  2000/07/08 16:28:05  gerd
- * 	Updated: Exception 'Not_resolvable' is taken into account.
- *
- * Revision 1.2  2000/07/04 22:12:47  gerd
- * 	Update: Case ext_id = Anonymous.
- * 	Update: Handling of the exception Not_competent when reading
- * from a resolver.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_entity.ml:
- *
- * Revision 1.27  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.26  2000/05/28 17:24:55  gerd
- * 	Bugfixes.
- *
- * Revision 1.25  2000/05/27 19:23:32  gerd
- * 	The entities store whether they count as external with
- * respect to the standalone check: New methods counts_as_external
- * and set_counts_as_external.
- * 	The entity manager can find out whether the current
- * entity counts as external: method current_entity_counts_as_external.
- *
- * Revision 1.24  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.23  2000/05/14 21:51:24  gerd
- * 	Change: Whitespace is handled by the grammar, and no longer
- * by the entity.
- *
- * Revision 1.22  2000/05/14 17:50:54  gerd
- * 	Updates because of changes in the token type.
- *
- * Revision 1.21  2000/05/09 00:02:44  gerd
- * 	Conditional sections are now recognized by the parser.
- * There seem some open questions; see the TODO comments!
- *
- * Revision 1.20  2000/05/08 21:58:22  gerd
- * 	Introduced entity_manager as communication object between
- * the parser and the currently active entity.
- * 	New hooks handle_bof and handle_eof.
- * 	Removed "delegated entities". The entity manager contains
- * the stack of open entities.
- * 	Changed the way Begin_entity and End_entity are inserted.
- * This is now done by handle_bof and handle_eof.
- * 	The XML declaration is no longer detected by the entity.
- * This is now done by the parser.
- *
- * Revision 1.19  2000/05/01 15:18:44  gerd
- * 	Improved CRLF handling in the replacement text of entities.
- * 	Changed one error message.
- *
- * Revision 1.18  2000/04/30 18:18:39  gerd
- * 	Bugfixes: The conversion of CR and CRLF to LF is now hopefully
- * done right. The new variable "normalize_newline" indicates whether
- * normalization must happen for that type of entity. The normalization
- * if actually carried out separately for every token that needs it.
- *
- * Revision 1.17  2000/03/13 23:42:38  gerd
- * 	Removed the resolver classes, and put them into their
- * own module (Markup_reader).
- *
- * Revision 1.16  2000/02/22 01:06:58  gerd
- * 	Bugfix: Resolvers are properly re-initialized. This bug caused
- * that entities could not be referenced twice in the same document.
- *
- * Revision 1.15  2000/01/20 20:54:11  gerd
- * 	New config.errors_with_line_numbers.
- *
- * Revision 1.14  2000/01/08 18:59:03  gerd
- * 	Corrected the string resolver.
- *
- * Revision 1.13  1999/09/01 22:58:23  gerd
- * 	Method warn_not_latin1 raises Illegal_character if the character
- * does not match the Char production.
- * 	External entities that are not document entities check if the
- * <?xml...?> declaration at the beginning matches the TextDecl production.
- * 	Method xml_declaration has type ... list option, not ... list.
- * 	Tag_beg and Tag_end now carry an entity_id with them.
- * 	The code to check empty entities has changed. That the Begin_entity/
- * End_entity pair is not to be added must be explicitly turned on. See the
- * description of empty entity handling in design.txt.
- * 	In internal subsets entity declarations are not allowed to refer
- * to parameter entities. The internal_entity class can do this now.
- * 	The p_parsed parameter of internal_entity has gone. It was simply
- * superflous.
- *
- * Revision 1.12  1999/09/01 16:24:13  gerd
- * 	The method replacement_text returns the text as described for
- * "included in literal". The former behaviour has been dropped to include
- * a leading and a trailing space character for parameter entities.
- * 	Bugfix: When general entities are included, they are always parsed.
- *
- * Revision 1.11  1999/08/31 19:13:31  gerd
- * 	Added checks on proper PE nesting. The idea is that tokens such
- * as Decl_element and Decl_rangle carry an entity ID with them. This ID
- * is simply an object of type < >, i.e. you can only test on identity.
- * The lexer always produces tokens with a dummy ID because it does not
- * know which entity is the current one. The entity layer replaces the dummy
- * ID with the actual ID. The parser checks that the IDs of pairs such as
- * Decl_element and Decl_rangle are the same; otherwise a Validation_error
- * is produced.
- *
- * Revision 1.10  1999/08/19 01:06:41  gerd
- * 	Improved error messages: external entities print their
- * ext id, too
- *
- * Revision 1.9  1999/08/15 20:35:48  gerd
- * 	Improved error messages.
- * 	Before the tokens Plus, Star, Qmark space is not allowed any longer.
- * 	Detection of recursive entity references is a bit cleaner.
- *
- * Revision 1.8  1999/08/15 15:33:44  gerd
- * 	Revised whitespace checking: At certain positions there must be
- * white space. These checks cannot be part of the lexer, as %entity; counts
- * as white space. They cannot be part of the yacc parser because one look-ahead
- * token would not suffice if we did that. So these checks must be done by the
- * entity layer. Luckily, the rules are simple: There are simply a number of
- * token pairs between which white space must occur independently of where
- * these token have been found. Two variables, "space_seen", and "last_token"
- * have been added in order to check these rules.
- *
- * Revision 1.7  1999/08/15 00:41:06  gerd
- * 	The [ token of conditional sections is now allowed to occur
- * in a different entity.
- *
- * Revision 1.6  1999/08/15 00:29:02  gerd
- * 	The method "attlist_replacement_text" has gone. There is now a
- * more general "replacement_text" method that computes the replacement
- * text for both internal and external entities. Additionally, this method
- * returns whether references to external entities have been resolved;
- * this is checked in the cases where formerly "attlist_replacement_text"
- * was used as it is not allowed everywhere.
- * 	Entities have a new slot "need_spaces" that indicates that the
- * next token must be white space or a parameter reference. The problem
- * was that "<!ATTLIST%e;" is legal because when including parameter
- * entities white space is added implicitly. Formerly, the white space
- * was expected by the underlying lexer; now the lexer does not check
- * anymore that "<!ATTLIST" is followed by white space because the lexer
- * cannot handle parameter references. Because of this, the check on
- * white space must be done by the entity.
- *
- * Revision 1.5  1999/08/14 22:57:19  gerd
- * 	It is allowed that external entities are empty because the
- * empty string is well-parsed for both declarations and contents. Empty
- * entities can be referenced anywhere because the references are replaced
- * by nothing. Because of this, the Begin_entity...End_entity brace is only
- * inserted if the entity is non-empty. (Otherwise references to empty
- * entities would not be allowed anywhere.)
- * 	As a consequence, the grammar has been changed such that a
- * single Eof is equivalent to Begin_entity,End_entity without content.
- *
- * Revision 1.4  1999/08/14 22:11:19  gerd
- *         Several objects have now a "warner" as argument which is
- * an object with a "warn" method. This is used to warn about characters
- * that cannot be represented in the Latin 1 alphabet.
- * 	Previously, the resolvers had features in order to warn about
- * such characters; this has been removed.
- * 	UTF-8 streams can be read even if they contain characters
- * that cannot be represented by 16 bits.
- * 	The buffering used in the resolvers is now solved in a
- * cleaner way; the number of characters that are expected to be read
- * from a source can be limited. This removes a bug with UTF-16 streams
- * that previously lead to wrong exceptions; and the buffering is more
- * efficient, too.
- *
- * Revision 1.3  1999/08/11 14:58:53  gerd
- * 	Some more names for encodings are allowed, such as "utf8" instead
- * of the standard name "UTF-8".
- * 	'resolve_as_file' interprets relative file names as relative to
- * the "parent" resolver.
- *
- * Revision 1.2  1999/08/10 21:35:07  gerd
- * 	The XML/encoding declaration at the beginning of entities is
- * evaluated. In particular, entities have now a method "xml_declaration"
- * which returns the name/value pairs of such a declaration. The "encoding"
- * setting is interpreted by the entity itself; "version", and "standalone"
- * are interpreted by Markup_yacc.parse_document_entity. Other settings
- * are ignored (this does not conform to the standard; the standard prescribes
- * that "version" MUST be given in the declaration of document; "standalone"
- * and "encoding" CAN be declared; no other settings are allowed).
- * 	TODO: The user should be warned if the standard is not exactly
- * fulfilled. -- The "standalone" property is not checked yet.
- *
- * Revision 1.1  1999/08/10 00:35:51  gerd
- * 	Initial revision.
- *
- *
- *)
