@@ -1,4 +1,4 @@
-(* $Id: pxp_reader.mli,v 1.15 2003/06/21 13:17:30 gerd Exp $
+(* $Id: pxp_reader.mli 689 2004-08-07 17:01:52Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -33,6 +33,12 @@ exception Not_resolvable of exn;;
    * Not_resolvable(Not_found) serves as indicator for an unknown reason.
    *)
 
+(* One must only use either [lsrc_lexbuf], or [lsrc_unicode_lexbuf] ! *)
+type lexer_source =
+    { lsrc_lexbuf : Lexing.lexbuf Lazy.t;
+      lsrc_unicode_lexbuf : Netulex.ULB.unicode_lexbuf Lazy.t;
+    }
+
 
 (* The class type 'resolver' is the official type of all "resolvers".
  * Resolvers take file names (or better, external identifiers) and
@@ -154,16 +160,20 @@ class type resolver =
 
     method rep_encoding : rep_encoding
 
-    method open_in : ext_id -> Lexing.lexbuf
+    method open_in : ext_id -> lexer_source
       (* This is the old method to open a resolver. It is superseded by
        * open_rid.
        * This method may raise Not_competent if the object does not know
        * how to handle this ext_id.
+       *
+       * PXP 1.2: Returns now a lexer_source, no longer a lexbuf
        *)
 
-    method open_rid : resolver_id -> Lexing.lexbuf
+    method open_rid : resolver_id -> lexer_source
       (* This is the new method to open a resolver. It takes a resolver ID
        * instead of an ext_id but works in the same way.
+       *
+       * PXP 1.2: Returns now a lexer_source, no longer a lexbuf
        *)
 
     method close_in : unit
@@ -340,6 +350,7 @@ class resolve_as_file :
   ?map_private_id:  (private_id -> Neturl.url) ->
   ?open_private_id: (private_id -> in_channel * encoding option) ->
   ?base_url_defaults_to_cwd: bool ->
+  ?not_resolvable_if_not_found:bool ->
   unit ->
   resolver;;
 
@@ -377,13 +388,16 @@ class resolve_as_file :
    * from and to get the character encoding. The URL is taken into account
    * when subsequently relative SYSTEM IDs must be resolved.
    *
-   * Option ~base_url_defaults_to_cwd: If true (the default), relative URLs
+   * Option ~base_url_defaults_to_cwd: If true, relative URLs
    * are interpreted relative to the current working directory at the time
    * the class is instantiated, but only if there is no parent URL, i.e.
-   * rid_system_base=None. If false, such URLs cannot be resolved. This 
-   * option is selected by default because of backward compatibility. 
+   * rid_system_base=None. If false (the default), such URLs cannot be resolved.
    * In general, it is better to set this option to false, and to
    * initialize rid_system_base properly.
+   *
+   * Option ~not_resolvable_if_not_found: If true (the default), 
+   * "File not found" errors stop the resolution process. If false,
+   * "File not found" is treated as [Not_competent].
    *)
 
 val make_file_url :
@@ -418,6 +432,9 @@ class lookup_id :
   (* The general catalog class. The catalog argument specifies pairs (xid,r)
    * mapping external IDs xid to subresolvers r. The subresolver is invoked
    * if an entity with the corresponding xid is to be opened.
+   *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
    *)
 
 
@@ -430,6 +447,9 @@ class lookup_id_as_file :
    * to files. The file is read  if an entity with the corresponding xid is
    * to be opened.
    *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
+   *
    * ~fixenc: Overrides the encoding of the file contents. By default, the
    *     standard rule is applied to find out the encoding of the file.
    *)
@@ -443,6 +463,9 @@ class lookup_id_as_string :
   (* The catalog argument specifies pairs (xid,s) mapping external IDs xid
    * to strings s. The string is read if an entity with the corresponding
    * xid is to be opened.
+   *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
    *)
 
 
@@ -453,6 +476,7 @@ class lookup_public_id :
   (* This is the generic builder for PUBLIC id catalog resolvers: The catalog 
    * argument specifies pairs (pubid, r) mapping PUBLIC identifiers to
    * subresolvers.
+   *
    * The subresolver is invoked if an entity with the corresponding PUBLIC
    * id is to be opened.
    *)
@@ -469,6 +493,13 @@ class lookup_public_id_as_file :
    * filenames must already be encoded in the character set the system uses
    * for filenames.
    *
+   * Note: This class does not enable the resolution of inner IDs of PUBLIC
+   * entities by relative SYSTEM names. To get this effect, use
+   * the class lookup_id, and feed it with combined 
+   * Public(pubid,sysid) identifiers. In this case, the entity has both
+   * a PUBLIC and a SYSTEM ID, and resolution of inner relative SYSTEM
+   * names works.
+   *
    * ~fixenc: Overrides the encoding of the file contents. By default, the
    *     standard rule is applied to find out the encoding of the file.
    *)
@@ -500,7 +531,10 @@ class lookup_system_id :
    * Important note: Two SYSTEM IDs are considered as equal if they are
    * equal in their string representation. (This may not what you want
    * and may cause trouble... However, I currently do not know how to
-   * implement a "semantic" comparison logic.)
+   * implement a "semantical" comparison logic.)
+   *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
    *)
 
 
@@ -514,6 +548,9 @@ class lookup_system_id_as_file :
    * filenames must already be encoded in the character set the system uses
    * for filenames.
    *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
+   *
    * ~fixenc: Overrides the encoding of the file contents. By default, the
    *     standard rule is applied to find out the encoding of the file.
    *)
@@ -528,6 +565,9 @@ class lookup_system_id_as_string :
    * pairs (sysid, text) mapping SYSTEM identifiers to XML text (which must
    * begin with <?xml ...?>).
    *
+   * Note: SYSTEM IDs are simply compared literally, without making
+   * relative IDs absolute. See norm_system_id below for this function.
+   *
    * ~fixenc: Overrides the encoding of the strings.
    *)
 
@@ -539,7 +579,7 @@ class norm_system_id : resolver -> resolver
    * Normalization includes:
    * - Relative URLs are made absolute. If this fails, the problematic
    *   relative URL will be rejected.
-   * - .. and . and // are removed 
+   * - .. and . and // in the middle of URLs are removed 
    * - Escaping of reserved characters is normalized
    *
    * Normalization is recommended for catalogs, e.g.
@@ -580,7 +620,8 @@ class rewrite_system_id :
    * The class normalizes URLs as norm_system_id does, before the match
    * is tried.
    *
-   * By default, URLs that do not match any pattern are rejected.
+   * By default, URLs that do not match any pattern are rejected
+   * (Not_competent).
    *
    * The rewritten URL is only visible within the passed subresolver.
    * If the opened entity accesses other entities by relative URLs,
@@ -618,7 +659,6 @@ type combination_mode =
 
 
 class combine : 
-        ?prefer:resolver -> 
 	?mode:combination_mode ->
 	resolver list -> 
 	  resolver;;
@@ -642,8 +682,6 @@ class combine :
    * resolvers are cloned and again combined. If the 'clone' method is
    * invoked after 'open_rid' (i.e. while the resolver is open), only the
    * active resolver is cloned.
-   *
-   * ~prefer: This is an internally used option.
    *)
 
 
@@ -905,96 +943,3 @@ val lookup_system_id_as_string :
     resolver;;
   (* Same as the equally named class *)
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_reader.mli,v $
- * Revision 1.15  2003/06/21 13:17:30  gerd
- * 	Fixes regarding Anonymous.
- *
- * Revision 1.14  2003/06/20 15:14:14  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.13  2003/06/19 21:51:12  gerd
- * 	Removed the close_all method. It is no longer necessary, because
- * we have now the entity_manager.
- * 	Several smaller fixes.
- *
- * Revision 1.12  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.11  2003/02/01 15:02:09  gerd
- * 	New: norm_system_id, rewrite_system_id.
- * 	Fixed: The empty system name is regarded as "NIL" value. It
- * never matches with itself.
- *
- * Revision 1.10  2003/01/21 00:18:48  gerd
- * 	Reimplementation of the reader classes, now basing on the
- * resolver_id type to identify entities.
- *
- * Revision 1.9  2001/07/01 08:35:23  gerd
- * 	Instead of the ~auto_close argument, there is now a
- * ~close argument for several functions/classes. This allows some
- * additional action when the resolver is closed.
- *
- * Revision 1.8  2001/04/22 14:16:48  gerd
- * 	resolve_as_file: you can map private IDs to arbitrary channels.
- * 	resolve_read_url_channel: changed type of the channel_of_url
- * argument (ext_id is also passed)
- * 	More examples and documentation.
- *
- * Revision 1.7  2001/04/03 20:22:44  gerd
- * 	New resolvers for catalogs of PUBLIC and SYSTEM IDs.
- * 	Improved "combine": PUBLIC and SYSTEM IDs are handled
- * separately.
- * 	Rewritten from_file: Is now a simple application of the
- * Pxp_reader classes and functions. (The same has still to be done
- * for from_channel!)
- *
- * Revision 1.6  2001/02/01 20:38:49  gerd
- * 	New support for PUBLIC identifiers.
- *
- * Revision 1.5  2000/07/09 01:05:33  gerd
- * 	New methode 'close_all' that closes the clones, too.
- *
- * Revision 1.4  2000/07/08 16:24:56  gerd
- * 	Introduced the exception 'Not_resolvable' to indicate that
- * 'combine' should not try the next resolver of the list.
- *
- * Revision 1.3  2000/07/06 23:04:46  gerd
- * 	Quick fix for 'combine': The active resolver is "prefered",
- * but the other resolvers are also used.
- *
- * Revision 1.2  2000/07/04 22:06:49  gerd
- * 	MAJOR CHANGE: Complete redesign of the reader classes.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_reader.mli:
- *
- * Revision 1.3  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.2  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.1  2000/03/13 23:41:54  gerd
- * 	Initial revision.
- *
- *
- *)
