@@ -1,4 +1,4 @@
-(* $Id: pxp_tree_parser.ml,v 1.6 2003/11/04 22:45:07 gerd Exp $
+(* $Id: pxp_tree_parser.ml 707 2004-09-04 17:25:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -245,6 +245,11 @@ object (self)
       early_material <- [];
       (* Move the super root or the emulation to the stack: *)
       stack_push (super_root, "", (self :> entity_id)) elstack;
+      (* Init namespace processing, if necessary: *)
+      ( match config.enable_namespace_processing with
+	    None -> ()
+	  | Some mng -> self # init_ns_processing mng
+      );
       init_done <- true;
     end
 
@@ -311,14 +316,9 @@ object (self)
 		spec dtd norm_name norm_attlist
 	    in
 
-	    if config.enable_namespace_info then begin
-	      let info =
-		new namespace_info_impl
-		      src_prefix
-		      element
-		      ( ("!", default_normprefix) :: src_norm_mapping) in
-	      element # set_namespace_info (Some info);
-	    end;
+	    let scope = match ns_scope with Some s -> s | None -> assert false 
+	    in
+	    element # set_namespace_scope scope;
 
 	    element
 
@@ -422,7 +422,7 @@ object (self)
     else
       current_data <- data :: current_data
 
-  method private event_pinstr position target value =
+  method private event_pinstr position target value ent_id =
     (* position: The position of the processing instruction
      * target: The name following <?
      * value: The string following the name
@@ -767,29 +767,3 @@ let extract_dtd_from_document_entity cfg src =
 
 === *)
 
-(* ======================================================================
- * History:
- * 
- * $Log: pxp_tree_parser.ml,v $
- * Revision 1.6  2003/11/04 22:45:07  gerd
- * 	Problems with Lexing under 3.07 fixed
- *
- * Revision 1.5  2003/06/29 15:44:30  gerd
- * 	New entry flag: `Val_mode_dtd
- *
- * Revision 1.4  2003/06/20 21:00:33  gerd
- * 	Moved events to Pxp_types.
- * 	Implementation of namespaces in event-based parsers.
- *
- * Revision 1.3  2003/06/20 19:41:39  gerd
- * 	Added ~transform_dtd to parse_wfdocument_entity.
- *
- * Revision 1.2  2003/06/20 15:14:14  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.1  2003/06/15 18:18:34  gerd
- * 	Initial revision
- *
- * 
- *)
