@@ -1,4 +1,4 @@
-(* $Id: pxp_types.mli,v 1.23 2003/10/20 20:35:24 gerd Exp $
+(* $Id: pxp_types.mli 707 2004-09-04 17:25:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -126,7 +126,6 @@ type config =
 	  *     T_element "-vr" instead of T_super_root.
 	  * (2) The T_super_root node is created from the super root exemplar
 	  *     in your spec.
-	  * (3) Event-based parser: no effect
 	  *)
 
       enable_comment_nodes : bool;
@@ -338,18 +337,6 @@ type config =
 	 *    E_end_tag, respectively.
 	 *)
 
-      enable_namespace_info : bool;
-        (* Whether to set the namespace_info slot of elements. 
-	 * This option has only an effect if enable_namespace_processing is
-	 * non-None.
-	 *
-	 * Warning! This option requires a lot of memory!
-	 *
-	 * Default: false
-	 *
-	 * Event-based parser: this option is ignored.
-	 *)
-
       (* Experimental stuff: *)
 
       escape_contents : 
@@ -649,52 +636,54 @@ type entry =
 
 
 type event =
-  | E_start_doc of (string * bool * Pxp_dtd.dtd)
-  | E_end_doc
+  | E_start_doc of (string * Pxp_dtd.dtd)
+  | E_end_doc of string
   | E_start_tag of (string * (string * string) list * 
+		    Pxp_dtd.namespace_scope option *
 		    Pxp_lexer_types.entity_id)
-  | E_ns_start_tag of (string * string * (string * string * string) list *
-		       Pxp_lexer_types.entity_id)
   | E_end_tag    of (string * Pxp_lexer_types.entity_id)
-  | E_ns_end_tag of (string * string * Pxp_lexer_types.entity_id)
   | E_char_data of  string
-  | E_pinstr of (string * string)
+  | E_pinstr of (string * string * Pxp_lexer_types.entity_id)
+  | E_pinstr_member of (string * string * Pxp_lexer_types.entity_id)
   | E_comment of string
+  | E_start_super
+  | E_end_super
   | E_position of (string * int * int)
   | E_error of exn
   | E_end_of_stream
   (* may be extended in the future *)
 
   (* The type of XML events:
-   * E_start_doc (xmlversion,standalone,dtd)
-   * E_end_doc
+   * E_start_doc (xmlversion,dtd)
+   * E_end_doc lit_name
+   *    lit_name: The literal name of the root element
    *
-   * E_start_tag (name, attlist, entid):
+   * E_start_tag (name, attlist, scope_opt, entid):
    *    <name attlist>
-   *    only used in non-namespace mode
-   *
-   * E_ns_start_tag (orig_name, norm_name, attlist, entid)
-   *    only used in namespace mode; orig_name is the element as found
-   *    in the XML text; norm_name is the normalized element name;
-   *    attlist consists of triples (orig_name, norm_name, value).
+   *    scope_opt is None in non-namespace mode, and the
+   *    namespace scope object in namespace mode.
    *
    * E_end_tag (name, entid):                 
    *    </name>
-   *    only used in non-namespace mode
-   *
-   * E_ns_end_tag (orig_name, norm_name, entid):
-   *    only used in namespace mode
    *
    * E_char_data data:
    *     The parser usually generates several E_char_data events for a
    *     longer section of character data.
    *
    * E_pinstr (target,value):                 
-   *     <?target value?>
+   *     <?target value?> as node
+   *
+   * E_pinstr_member (target,value):                 
+   *     <?target value?> as member of the parent element (add_pinstr)
    *
    * E_comment value:
    *     <!--value-->
    *
+   * E_start_super, 
+   * E_end_super:
+   *    Indicates where the "super root node" is. Only generated when
+   *    enable_super_root_node is on.
+   *
    * E_position(entity,line,col):
    *    these events are only created if the next event will be
    *    E_start_tag, E_pinstr, or E_comment, and if
@@ -707,131 +696,15 @@ type event =
    * E_error(exn):
    *    this last event indicates that the parser has terminated with
    *    error
+   *
+   * Note Pxp_lexer_types.entity_id: currently, this is just < >, i.e.
+   * the class type without properties. It is planned, however, that
+   * one can at least query the base URI of the entity. The best
+   * way of dealing with this parameter for now:
+   * - When parsing events, ignore it
+   * - When creating events, use Pxp_dtd.Entity.create_entity_id to 
+   *   generate new IDs.
    *)
 
 
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_types.mli,v $
- * Revision 1.23  2003/10/20 20:35:24  gerd
- * 	Updated docs (zack 2003-10-20)
- *
- * Revision 1.22  2003/06/29 15:44:30  gerd
- * 	New entry flag: `Val_mode_dtd
- *
- * Revision 1.21  2003/06/20 21:00:33  gerd
- * 	Moved events to Pxp_types.
- * 	Implementation of namespaces in event-based parsers.
- *
- * Revision 1.20  2003/06/20 15:14:14  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.19  2003/06/19 21:10:15  gerd
- * 	Revised the from_* functions.
- *
- * Revision 1.18  2003/06/15 18:19:56  gerd
- * 	Pxp_yacc has been split up
- *
- * Revision 1.17  2003/06/15 12:23:22  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.16  2003/01/21 00:18:09  gerd
- * 	New type resolver_id. It is related to ext_id but contains
- * more information.
- *
- * Revision 1.15  2002/08/28 23:54:34  gerd
- * 	Support for new lexer definition style.
- *
- * Revision 1.14  2001/06/27 23:33:53  gerd
- * 	Type output_stream is now a polymorphic variant
- *
- * Revision 1.13  2001/06/07 22:49:51  gerd
- * 	New namespace exceptions.
- *
- * Revision 1.12  2001/04/26 23:57:05  gerd
- * 	New exception Method_not_applicable. It is raised if there are
- * classes A and B both conforming to class type C, but A does not implement
- * a method required by the class type. In this case, invoking the method
- * in A raises Method_not_applicable.
- * 	This feature is mainly used in Pxp_document.
- *
- * Revision 1.11  2001/04/22 14:14:41  gerd
- * 	Updated to support private IDs.
- *
- * Revision 1.10  2001/02/01 20:37:38  gerd
- * 	Changed comment.
- *
- * Revision 1.9  2000/09/09 16:38:47  gerd
- * 	New type 'pool'.
- *
- * Revision 1.8  2000/08/14 22:24:55  gerd
- * 	Moved the module Pxp_encoding to the netstring package under
- * the new name Netconversion.
- *
- * Revision 1.7  2000/07/27 00:41:15  gerd
- * 	new 8 bit codes
- *
- * Revision 1.6  2000/07/16 18:31:09  gerd
- * 	The exception Illegal_character has been dropped.
- *
- * Revision 1.5  2000/07/16 16:34:21  gerd
- * 	Updated comments.
- *
- * Revision 1.4  2000/07/14 21:25:27  gerd
- * 	Simplified the type 'collect_warnings'.
- *
- * Revision 1.3  2000/07/08 16:23:50  gerd
- * 	Added the exception 'Error'.
- *
- * Revision 1.2  2000/07/04 22:08:26  gerd
- * 	type ext_id: New variant Anonymous. - The System and Public
- * variants are now encoded as UTF-8.
- * 	collect_warnings is now a class type only. New class
- * drop_warnings.
- * 	New functions  encoding_of_string and string_of_encoding.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from Markup_types.mli:
- *
- * Revision 1.7  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.6  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.5  2000/05/01 20:43:25  gerd
- *         New type output_stream; new function 'write'.
- *
- * Revision 1.4  1999/09/01 16:25:35  gerd
- * 	Dropped Illegal_token and Content_not_allowed_here. WF_error can
- * be used instead.
- *
- * Revision 1.3  1999/08/15 02:22:40  gerd
- *         Added exception Undeclared.
- *
- * Revision 1.2  1999/08/14 22:15:17  gerd
- *         New class "collect_warnings".
- *
- * Revision 1.1  1999/08/10 00:35:52  gerd
- * 	Initial revision.
- *
- *
- *)
